@@ -1,6 +1,6 @@
 # Author:: Nathaniel Talbott.
 # Copyright:: Copyright (c) 2000-2003 Nathaniel Talbott. All rights reserved.
-#             Copyright (c) 2009-2010 Kouhei Sutou. All rights reserved.
+#             Copyright (c) 2009-2012 Kouhei Sutou. All rights reserved.
 # License:: Ruby license.
 
 require 'test/unit/assertionfailederror'
@@ -85,6 +85,47 @@ module Test
         end
       end
 
+      # Asserts that +object+ is false or nil.
+      #
+      # @note Just for minitest compatibility. :<
+      #
+      # @param [Object] object The object to be asserted.
+      # @return [void]
+      #
+      # @example Pass patterns
+      #   refute(false)    # => pass
+      #   refute(nil)      # => pass
+      #
+      # @example Fialure patterns
+      #   refute(true)     # => failure
+      #   refute("string") # => failure
+      #
+      # @since 2.5.3
+      def refute(object, message=nil)
+        _wrap_assertion do
+          assertion_message = nil
+          case message
+          when nil, String, Proc
+          when AssertionMessage
+            assertion_message = message
+          else
+            error_message = "assertion message must be String, Proc or "
+            error_message << "#{AssertionMessage}: "
+            error_message << "<#{message.inspect}>(<#{message.class}>)"
+            raise ArgumentError, error_message, filter_backtrace(caller)
+          end
+          assert_block("refute should not be called with a block.") do
+            !block_given?
+          end
+          assertion_message ||= build_message(message,
+                                              "<?> is neither nil or false.",
+                                              object)
+          assert_block(assertion_message) do
+            not object
+          end
+        end
+      end
+
       ##
       # Passes if +expected+ == +actual+.
       #
@@ -163,15 +204,8 @@ EOT
         _assert_raise(assert_expected_exception, *args, &block)
       end
 
-      ##
-      # Alias of assert_raise.
-      #
-      # Will be deprecated in 1.9, and removed in 2.0.
-
-      public
-      def assert_raises(*args, &block)
-        assert_raise(*args, &block)
-      end
+      # Just for minitest compatibility. :<
+      alias_method :assert_raises, :assert_raise
 
       ##
       # Passes if the block raises one of the given
@@ -344,6 +378,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_respond_to, :assert_not_respond_to
+
       ##
       # Passes if +string+ =~ +pattern+.
       #
@@ -466,6 +505,11 @@ EOT
         assert_block(full_message) { !actual.equal?(expected) }
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_same, :assert_not_same
+
       ##
       # Passes if +expected+ != +actual+
       #
@@ -478,6 +522,11 @@ EOT
         assert_block(full_message) { expected != actual }
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_equal, :assert_not_equal
+
       ##
       # Passes if ! +object+ .nil?
       #
@@ -490,6 +539,11 @@ EOT
         assert_block(full_message){!object.nil?}
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_nil, :assert_not_nil
+
       ##
       # Passes if +regexp+ !~ +string+
       #
@@ -510,6 +564,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_match, :assert_not_match
+
       ##
       # Deprecated. Use #assert_not_match instead.
       #
@@ -578,13 +637,10 @@ EOT
         end
       end
 
-      ##
-      # Alias of assert_throw.
+      # Just for minitest compatibility. :<
       #
-      # Will be deprecated in 1.9, and removed in 2.0.
-      def assert_throws(*args, &block)
-        assert_throw(*args, &block)
-      end
+      # @since 2.5.3
+      alias_method :assert_throws, :assert_throw
 
       ##
       # Passes if block does not throw anything.
@@ -661,6 +717,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :refute_in_delta, :assert_not_in_delta
+
       # :stopdoc:
       private
       def _assert_in_delta_validate_arguments(expected_float,
@@ -762,6 +823,7 @@ EOT
             else
               delta = normalized_expected_float * epsilon.to_f
             end
+            delta = delta.abs
             (normalized_expected_float - actual_float.to_f).abs <= delta
           end
         end
@@ -1285,6 +1347,11 @@ EOT
         end
       end
 
+      # Just for minitest compatibility. :<
+      #
+      # @since 2.5.3
+      alias_method :assert_includes, :assert_include
+
       ##
       # Passes if +collection+ doesn't include +object+.
       #
@@ -1379,11 +1446,14 @@ EOT
         end
       end
 
-      ##
-      # Called whenever an assertion is made.  Define this in classes that
-      # include Test::Unit::Assertions to record assertion counts.
-
-      private
+      public
+      # Called whenever an assertion is made. Define this in classes
+      # that include Test::Unit::Assertions to record assertion
+      # counts.
+      #
+      # This is a public API for developers who extend test-unit.
+      #
+      # @return [void]
       def add_assertion
       end
 
@@ -1615,7 +1685,7 @@ EOT
 
           class << self
             def target?(object)
-              object.is_a?(Hash) or object == ENV
+              object.is_a?(Hash) or ENV.equal?(object)
             end
           end
 
@@ -1769,7 +1839,7 @@ EOT
           private
           if Object.const_defined?(:Encoding)
             def encoding_safe_concat(buffer, parameter)
-              if Encoding.compatible?(buffer.encoding, parameter.encoding)
+              if Encoding.compatible?(buffer, parameter)
                 buffer << parameter
               else
                 buffer << parameter.dup.force_encoding(buffer.encoding)
