@@ -1,4 +1,6 @@
 require 'test/unit/color-scheme'
+require 'test/unit/priority'
+require 'test/unit/attribute-matcher'
 require 'optparse'
 
 module Test
@@ -48,22 +50,31 @@ module Test
         def prepare(hook=Proc.new)
           PREPARE_HOOKS << hook
         end
-      end
 
-      def self.run(force_standalone=false, default_dir=nil, argv=ARGV, &block)
-        r = new(force_standalone || standalone?, &block)
-        r.base = default_dir
-        r.prepare
-        r.process_args(argv)
-        r.run
-      end
+        def run(force_standalone=false, default_dir=nil, argv=ARGV, &block)
+          r = new(force_standalone || standalone?, &block)
+          r.base = default_dir
+          r.prepare
+          r.process_args(argv)
+          r.run
+        end
 
-      def self.standalone?
-        return false unless("-e" == $0)
-        ObjectSpace.each_object(Class) do |klass|
-          return false if(klass < TestCase)
+        def standalone?
+          return false unless("-e" == $0)
+          ObjectSpace.each_object(Class) do |klass|
+            return false if(klass < TestCase)
+          end
+          true
+        end
+
+        @@need_auto_run = true
+        def need_auto_run?
+          @@need_auto_run
+        end
+
+        def need_auto_run=(need)
+          @@need_auto_run = need
         end
-        true
       end
 
       register_collector(:descendant) do |auto_runner|
@@ -117,7 +128,6 @@ module Test
       attr_writer :runner, :collector
 
       def initialize(standalone)
-        Unit.run = true
         @standalone = standalone
         @runner = default_runner
         @collector = default_collector
@@ -144,7 +154,6 @@ module Test
       end
 
       def process_args(args=ARGV)
-        args = args.dup
         begin
           args.unshift(*@default_arguments)
           options.order!(args) {|arg| @to_run << arg}
@@ -251,6 +260,36 @@ module Test
             end
           end
 
+          o.on('--location=LOCATION', String,
+               "Runs tests that defined in LOCATION.",
+               "LOCATION is one of PATH:LINE, PATH or LINE") do |location|
+            if /\A\d+\z/ =~ location
+              path = nil
+              line = location.to_i
+            else
+              path, line, = location.split(/:(\d+)/, 2)
+              line = line.to_i unless line.nil?
+            end
+            @filters << lambda do |test|
+              test.class.test_defined?(:path => path,
+                                       :line => line,
+                                       :method_name => test.method_name)
+            end
+          end
+
+          o.on('--attribute=EXPRESSION', String,
+               "Runs tests that matches EXPRESSION.",
+               "EXPRESSION is evaluated as Ruby's expression.",
+               "Test attribute name can be used with no receiver in EXPRESSION.",
+               "EXPRESSION examples:",
+               "  !slow",
+               "  tag == 'important' and !slow") do |expression|
+            @filters << lambda do |test|
+              matcher = AttributeMatcher.new(test)
+              matcher.match?(expression)
+            end
+          end
+
           priority_filter = Proc.new do |test|
             if @filters == [priority_filter]
               Priority::Checker.new(test).need_to_run?
@@ -357,15 +396,18 @@ module Test
       end
 
       def run
+        self.class.need_auto_run = false
         suite = @collector[self]
         return false if suite.nil?
+        return true if suite.empty?
         runner = @runner[self]
         return false if runner.nil?
         @runner_options[:color_scheme] ||= @color_scheme
         @runner_options[:listeners] ||= []
         @runner_options[:listeners].concat(@listeners)
-        Dir.chdir(@workdir) if @workdir
-        runner.run(suite, @runner_options).passed?
+        change_work_directory do
+          runner.run(suite, @runner_options).passed?
+        end
       end
 
       def load_config(file)
@@ -415,11 +457,18 @@ module Test
         file = global_config_file
         load_config(file) if file and File.exist?(file)
       end
+
+      def change_work_directory(&block)
+        if @workdir
+          Dir.chdir(@workdir, &block)
+        else
+          yield
+        end
+      end
     end
   end
 end
 
 require 'test/unit/runner/console'
 require 'test/unit/runner/emacs'
-require 'test/unit/runner/tap'
 require 'test/unit/runner/xml'
